'use strict';
const {Readable} = require('stream');

class HTTP2IncomingMessage extends Readable {
	constructor(req, response) {
		super();

		this.statusCode = response.headers[':status'];
		this.statusMessage = '';
		this.httpVersion = '2.0';
		this.httpVersionMajor = 2;
		this.httpVersionMinor = 0;
		this.headers = response.headers;
		this.trailers = [];
		this.req = req;

		this.aborted = false;
		this.complete = false;

		this.rawHeaders = response.rawHeaders;
		this.rawTrailers = [];

		const self = this;

		this.socket = req.session.socket;
		this.connection = req.session.socket;

		this.req._request.on('close', () => {
			if (!self.aborted) {
				self.complete = true;
			}
		});

		this._dumped = false;
	}

	_read() {
		if (this.req.session.destroyed) {
			this.push(null);
			return;
		}

		const self = this;
		const listener = chunk => {
			if (!self.push(chunk)) {
				self.req._request.removeListener('data', listener);
			}
		};

		this.req._request.on('data', listener);
	}

	destroy(error) {
		this.req.session.destroy(error);
	}

	setTimeout(ms, callback) {
		this.req.setTimeout(ms, callback);
		return this;
	}

	_dump() {
		if (!this._dumped) {
			this.dumped = true;
			this.req._request.removeAllListeners('data');
			this.req._request.resume();
		}
	}
}

module.exports = HTTP2IncomingMessage;
